/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.directory;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printNumbered;

import java.io.File;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.List;

import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.p2.bridge.IUIdentity;
import org.sonatype.p2.bridge.MetadataRepository;

import com.google.inject.BindingAnnotation;
import com.google.inject.Inject;

@Command( name = "p2/list/installable" )
public class ListInstallableCommand
    extends P2Command
{

    private final FileSystemAccess fileSystem;

    private final MetadataRepository metadataRepository;

    private final RepositoryRegistry repositoryRegistry;

    private final List<IUIdentity> latest;

    @Inject
    public ListInstallableCommand( final FileSystemAccess fileSystem, final MetadataRepository metadataRepository,
                                   final RepositoryRegistry repositoryRegistry,
                                   @Latest final List<IUIdentity> latest )
    {
        assert fileSystem != null;
        assert metadataRepository != null;
        assert repositoryRegistry != null;
        assert latest != null;

        this.fileSystem = fileSystem;
        this.metadataRepository = metadataRepository;
        this.repositoryRegistry = repositoryRegistry;
        this.latest = latest;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        if ( repositoryRegistry.isEmpty() )
        {
            io.error( "There should be at least one repository configured to be able to available units." );
            return Result.FAILURE;
        }

        final File location = getLocation( fileSystem );

        printHeader( io,
            String.format( "Available installable units for %s :", directory( location.getAbsolutePath() ) ) );

        latest.clear();

        if ( repositoryRegistry.isEmpty() )
        {
            io.error( "There should be at least one repository configured to be able to list installable units." );
            return Result.FAILURE;
        }

        final Collection<IUIdentity> ius = metadataRepository.getGroupIUs( repositoryRegistry.getAsURI() );

        if ( ius.size() == 0 )
        {
            io.println( "None" );
        }
        else
        {
            int i = 1;
            for ( final IUIdentity iu : ius )
            {
                printNumbered( io, i++, ius.size(), iu.toString() );
                latest.add( iu );
            }
        }

        io.println( "" );

        return Result.SUCCESS;
    }

    @Retention( RetentionPolicy.RUNTIME )
    @Target( { ElementType.FIELD, ElementType.PARAMETER } )
    @BindingAnnotation
    public @interface Latest
    {
    }

}
